#!/usr/bin/env python
""" compare.py : Used to compare an ARF or XCCDF result file to the expected results, as provided in a catalog file.
"""

__author__ = 'Matt Kerr'
__copyright__ = "Copyright 2011-2013, G2 Inc"
__credits__ = ["Matt Kerr", "Dragos Prisaca", "Melanie Cook"]
#__license__ = "TODO"
__version__ = "1.1"
__maintainer__ = 'Dragos Prisaca'
__email__ = "Dragos.Prisaca@g2-inc.com"
__status__ = "Beta"

import datetime
import hashlib
import xml.etree.ElementTree as etree
import argparse
import sys

class MyParser(argparse.ArgumentParser):
    def error(self, message):
        sys.stderr.write('error: %s\n' % message)
        self.print_help()
        sys.exit(2)

def printout(file = None, line = None):
    if file is not None:
        file.write(line)
    print(line)
    return

def hashFile(filename):
    file = open(filename, "rb")
    return hashlib.sha256(file.read()).hexdigest()

def addFileHeader(file=None, options=None):
    current_time = datetime.datetime.now()
    printout(file = file, line="========================================================================================================\n")
    printout(file = file, line="Date and Time:\t{0}\n".format(current_time.strftime("%Y-%m-%d %H:%M:%S")))
    printout(file = file, line="Catalog file:\t{0} (sha-256: {1})\n".format(options.catalogFile, hashFile(options.catalogFile)))
    printout(file = file, line="Results file:\t{0} (sha-256: {1})\n".format(options.resultsFile, hashFile(options.resultsFile)))
    if options.xccdfFileType is False:
        printout(file=file, line="Results Format:\tARF")
    else:
        printout(file=file, line="Results Format:\tXCCDF version {0}\n".format(options.xccdfVersion))
    printout(file = file, line="Configuration ID:\t{0}\n".format(options.configurationId))
    printout(file = file, line="========================================================================================================\n")
    return

def parseCatalogFile(catalogFile=None, configurationId = None, fileFormat = "arf", outfile=None):
    print("Loading the catalog file, this may take a while ... ")
    catalogTree = etree.parse(catalogFile)
    print("finished loading catalog file.\n")
    # locate the configuration section
    if configurationId is None:
        searchString = '//testSuite/expectedResults[@reportType="{0}"]'.format(fileFormat)
        configSection = catalogTree.find(searchString)
        pass
    else:
        searchString = './/testSuite[@suiteId="' + configurationId + '"]' + '/expectedResults[@reportType="{0}"]'.format(fileFormat)
        configSection = catalogTree.find(searchString)

    catalogResults = {}

    if configSection is not None:
        print("Locating expected results ...")
        for r in configSection:
            catalogResults[r.get('definition')] = r.get('expectedResult')
        print("expected results found.\n")
    else:
        printout(file = outfile, line="Error - Unable to find configuration id {0} in catalog file.\n".format(configurationId))
        return {}
    return catalogResults

def parseResultsFile(resultsFile=None, fileType="arf", fileVersion="1.2", outfile=None):
    actualResults = {}
    print("Parsing ARF results file, this may take a while ... ")
    tree = etree.parse(resultsFile)
    namespaceString = '{http://checklists.nist.gov/xccdf/' + fileVersion + '}'
    xstring = '//' + namespaceString + 'rule-result'
    results =  tree.findall(xstring)
    # read in the results
    for result in results:
        xstring = namespaceString + 'result'
        resultString = result.find(xstring)
        if result is not None:
            actualResults[result.get('idref')] = resultString.text
        else:
            actualResults[result.get('idref')] = "No result entry text"
    print("finished parsing ARF results file.\n")
    return actualResults

def compareResults(actualResults=None, expectedResults=None, outfile=None):
    printout(file = outfile, line="Starting comparison.\n")
    CountVariableMismatches=0
    CountVariableMissingResults=0
    for k,v in expectedResults.items():
        # check to see if the actual results have this item in them
        if v != "notselected":
            if k not in actualResults.keys():
                printout(file = outfile, line="The result file does not have an entry for expected item:{0}\n".format(k))
                CountVariableMissingResults += 1
            elif v != actualResults[k]:
                printout(file = outfile, line="The actual result {0} does not match the expected result of {1} for item:{2}\n".format(actualResults[k], v, k))
                CountVariableMismatches += 1
        # if expected result is 'notselected' -> the rule result may be missing from the XCCDF results or have value = 'notselected'
        elif k in actualResults.keys() and v != actualResults[k]:
            printout(file = outfile, line="The actual result {0} does not match the expected result of {1} for item:{2}\n".format(actualResults[k], v, k))
            CountVariableMismatches += 1
    for k,v in actualResults.items():
        if k not in expectedResults.keys():
            printout(file = outfile, line="The catalog file does not contain an entry for the item:{0}\n".format(k))
    printout(file = outfile, line="Comparison finished.\n")
    printout(file = outfile, line="==================\\\\ SUMMARY //==================\n")
    printout(file = outfile, line="Total number of mismatches:\t\t{0}.\n".format(CountVariableMismatches))
    printout(file = outfile, line="Total number of missing results:\t{0}.\n".format(CountVariableMissingResults))
    printout(file = outfile, line="=================================================\n")
    return

def main():
    # set up the command line argument processing
    epilogText = '''
    Examples:

    python compare.py ResultsFile CatalogFile -i ConfigurationId -o OutputFile
    Compares the ARF file specificied by ResultsFile with the values from ConfigurationId in CatalogFile and saves the results in OutputFile.

    python compare.py ResultsFile CatalogFile -x -xv 1.1
    Compares the XCCDF results file specified by ResultsFile to the values in the first configuration found in CatalogFile and outputs the result to the console.
    '''
    parser = MyParser(description='Compares ARF or XCCDF result files to expected results in a catalog.xml file.',
                      epilog=epilogText,
                      formatter_class=argparse.RawDescriptionHelpFormatter)
    #parser = argparse.ArgumentParser(description='Compares ARF or XCCDF result files to expected results in a catalog.xml file.', epilog='Epilog')
    parser.add_argument('resultsFile', help='Path and name of a file containing either ARF or XCCDF results')
    parser.add_argument('catalogFile', help="Path and name of a Catalog.xml file containing the expected results")
    parser.add_argument('-x', '--xccdf', dest="xccdfFileType", action='store_true', help="Indicates that the input file is an XCCDF results file")
    parser.add_argument('-xv', '--xccdfVersion', choices=['1.0','1.1','1.2'], dest="xccdfVersion", default='1.2', help='Specifies the XCCDF version of the results file. Defaults to 1.2 if not specified.')
    parser.add_argument('-i', '--configurationId', dest='configurationId', help="Configuration ID as specified in the catalog file.  If not specified the first configuration found will be used")
    parser.add_argument('-o', '--outputFile', dest='outputFile', help="Path and filename of the file to save the results in")
    parser.add_argument('-v', '--version', action='version', version='%(prog)s {0}'.format(__version__))
    options = parser.parse_args()

    resultsFile = options.resultsFile
    catalogFile = options.catalogFile
    configId = options.configurationId
    specVersion = options.xccdfVersion
    fileFormat = 'arf'
    # ARF only does XCCDF v 1.2 at this time
    if options.xccdfFileType is False:
        specVersion = '1.2'
        fileFormat = 'arf'
    else:
        fileFormat = 'xccdf'

    # if there is an output file specified open it
    outfile = None
    if options.outputFile is not None:
        outfile = open(options.outputFile, "w")
    else:
        print("No output file specified, output will be sent to the console only.\n")

    # print info
    addFileHeader(file=outfile, options=options)

    # open the catalog file
    expectedResults = parseCatalogFile(catalogFile=catalogFile, configurationId=configId, fileFormat=fileFormat, outfile=outfile)
    # open the arf results file
    actualResults = parseResultsFile(resultsFile = resultsFile, fileVersion=specVersion, outfile=outfile)
    #compare the results
    compareResults(actualResults=actualResults, expectedResults=expectedResults, outfile=outfile)

if __name__ == '__main__':
    main()



  